"""
Django settings for django_boilerplate project.

Generated by 'django-admin startproject' using Django 3.1.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""
import os
import sys
from pathlib import Path
from environ import Env

env = Env(
    # set casting, default value
    DEBUG=(bool, False),
    ADMIN_URL=(str, 'admin/'),
    TIME_ZONE=(str, 'Asia/Phnom_Penh'),
    SECRET_KEY=(str, 'S3cRétH3rë'),
    LANGUAGE_CODE=(str, 'en-us'),
    ALLOWED_HOSTS=(str, '* '),
    DJANGO_STATIC_HOST=(str, '')
)

Env.read_env()

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve(strict=True).parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env('SECRET_KEY')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env('DEBUG')

ALLOWED_HOSTS = env('ALLOWED_HOSTS').split(" ")

# Application definition

INSTALLED_APPS = [
    'whitenoise.runserver_nostatic',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Applications defined here
    'constance',
    'constance.backends.database',
    'core',
    'dj_database_url',
    'django_extensions',
    # Add Rest Framework with OAuth 2
    'rest_framework',
    'rest_framework.authtoken',
    'rest_auth',
    'oauth2_provider',
    'corsheaders',
    # Add Swagger for Rest Framework
    'drf_yasg',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'oauth2_provider.middleware.OAuth2TokenMiddleware'
]

ROOT_URLCONF = 'django_boilerplate.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'django_boilerplate.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    'default': env.db(),
    'extra': env.db('SQLITE_URL', default='sqlite:////tmp/db.sqlite3'),
}

CACHES = {
    'default': env.cache(),
    'redis': env.cache('REDIS_URL')
}

# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = env('LANGUAGE_CODE')

TIME_ZONE = env('TIME_ZONE')

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/

WHITENOISE_INDEX_FILE = True

CONSTANCE_BACKEND = 'constance.backends.database.DatabaseBackend'

CONSTANCE_ADDITIONAL_FIELDS = {
    'image_field': ['django.forms.ImageField', {}],
    'yes_no_null_select': ['django.forms.fields.ChoiceField', {
        'widget': 'django.forms.Select',
        'choices': ((None, "-----"), ("yes", "Yes"), ("no", "No"))
    }],
}

CONSTANCE_CONFIG = {
    'SITE_NAME': ('My Title', 'Website title'),
    'SITE_DESCRIPTION': ('My Description', 'Website description'),
    'LOGO_IMAGE': ('logo.png', 'Company logo', 'image_field'),
    'MY_SELECT_KEY': ('yes', 'select yes or no', 'yes_no_null_select'),
    'THEME': ('light-blue', 'Website theme'),
}

CONSTANCE_CONFIG_FIELDSETS = {
    'Site Options': {
        'fields': ('SITE_NAME', 'SITE_DESCRIPTION', 'LOGO_IMAGE', 'MY_SELECT_KEY'),
        'collapse': True
    },
    'Theme Options': ('THEME',),
}

STATIC_ROOT = os.path.join(BASE_DIR, 'staticfiles')
STATIC_HOST = env('DJANGO_STATIC_HOST')

STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

STATIC_URL = STATIC_HOST + '/static/'

REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "oauth2_provider.contrib.rest_framework.OAuth2Authentication",
        "rest_framework.authentication.BasicAuthentication",
        "rest_framework.authentication.TokenAuthentication",
        # "rest_framework.authentication.SessionAuthentication",
    ],
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated',
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly'
    ],
    "DEFAULT_RENDERER_CLASSES": [
        "rest_framework.renderers.JSONRenderer",
        "rest_framework.renderers.BrowsableAPIRenderer",
    ],
    "DEFAULT_PARSER_CLASSES": [
        "rest_framework.parsers.JSONParser",
        "rest_framework.parsers.FormParser",
        "rest_framework.parsers.MultiPartParser",
        "rest_framework.parsers.FileUploadParser",
    ],
    "DEFAULT_SCHEMA_CLASS": "rest_framework.schemas.coreapi.AutoSchema",
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "PAGE_SIZE": 2,
}

CORS_ORIGIN_ALLOW_ALL = True

OAUTH2_PROVIDER = {
    "SCOPES": {
        "read": "Read scope",
        "write": "Write scope",
        "admin": "Grands read and write access to administrative information"
    },
    # Set all request and authentication use JSON with Raw
    # "OAUTH2_BACKEND_CLASS": "oauth2_provider.oauth2_backends.JSONOAuthLibCore",
}

SWAGGER_SETTINGS = {
    "USE_SESSION_AUTH": False,
    "OAUTH2_CONFIG": {
        "appName": os.environ.get("API_NAME"),
        "clientId": os.environ.get("CLIENT_ID"),
        "clientSecret": os.environ.get("CLIENT_SECRET"),
    },
    "SECURITY_DEFINITIONS": {
        # "basic": {"type": "basic"},
        # "api_key": {"type": "apiKey", "name": "Authorization", "in": "header"},
        "oauth2": {
            "type": "oauth2",
            "flow": "password",
            "tokenUrl": os.environ.get("TOKEN_URL"),
            "scopes": OAUTH2_PROVIDER.get("SCOPES"),
        }
    },
    # "APIS_SORTER": "alpha",
    "DOC_EXPANSION": "list",
    "JSON_EDITOR": True,
    "OPERATIONS_SORTER": "method",
    "SHOW_REQUEST_HEADERS": True,
}



from django.urls import path, re_path, include
from drf_yasg import openapi
from drf_yasg.views import get_schema_view
from rest_framework import routers, permissions

from .views import PermissionViewSet, GroupViewSet, UserViewSet

router = routers.DefaultRouter()
router.register(r'permissions', PermissionViewSet)
router.register(r'groups', GroupViewSet)
router.register(r'users', UserViewSet)

schema_view = get_schema_view(
    openapi.Info(
        title="Boilerplate API",
        default_version='v1',
        description="Boilerplate API version 1.0.0",
        terms_of_service="https://www.google.com/policies/terms/",
        contact=openapi.Contact(email="adminz@gmail.com"),
        license=openapi.License(name="MIT License"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)

urlpatterns = [
    # AUTH 0 authentication
    path('auth0/', include('rest_auth.urls')),

    # OAUTH 2 authentication
    path('auth2/', include('oauth2_provider.urls', namespace='oauth2_provider')),

    # Rest Framework API support Browsable API
    path('', include(router.urls)),

    # Rest Framework Auth support Browsable API
    path('', include('rest_framework.urls', namespace='rest_framework')),

    re_path(r'^swagger(?P<format>\.json|\.yaml)$', schema_view.without_ui(cache_timeout=0), name='schema-json'),
    path('swagger/', schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
    path('redoc/', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc-ui'),
]



import os

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Renames the Django project"

    def add_arguments(self, parser):
        parser.add_argument(
            "old",
            type=str,
            nargs="+",
            help="The old Django project name"
        )
        parser.add_argument(
            "new",
            type=str,
            nargs="+",
            help="The new Django project name"
        )

    def handle(self, *args, **kwargs):
        old_project_name = kwargs["old"][0]
        new_project_name = kwargs["new"][0]

        files_to_rename = [
            f"{old_project_name}/settings.py",
            f"{old_project_name}/wsgi.py",
            f"{old_project_name}/asgi.py",
            f"{old_project_name}/urls.py",
            "manage.py",
        ]

        for f in files_to_rename:
            with open(f, "r") as file:
                content = file.read()

            content = content.replace(old_project_name, new_project_name)

            with open(f, "w") as file:
                file.write(content)

        os.rename(old_project_name, new_project_name)

        self.stdout.write(
            self.style.SUCCESS("Project {} has been renamed to {}".format(old_project_name, new_project_name))
        )



from django.contrib.auth.models import Permission, Group, User
from rest_framework import viewsets

from rest.serializers import PermissionSerializer, GroupSerializer, UserSerializer


class PermissionViewSet(viewsets.ModelViewSet):
    queryset = Permission.objects.all()
    serializer_class = PermissionSerializer


class GroupViewSet(viewsets.ModelViewSet):
    queryset = Group.objects.all()
    serializer_class = GroupSerializer


class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.all()
    serializer_class = UserSerializer



from django.core.management.base import BaseCommand
from django.utils import timezone


class Command(BaseCommand):
    help = 'Displays current date & time'

    def handle(self, *args, **kwargs):
        time = timezone.localtime().strftime("%A - %d/%m/%Y, %H:%M:%S %p")
        self.stdout.write(self.style.SUCCESS("Today is {}".format(time)))



from django.contrib.auth.models import Permission, Group, User

from rest_framework import serializers


class PermissionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Permission
        fields = ['url', 'id', 'name', 'codename']


class GroupSerializer(serializers.ModelSerializer):
    permissions = serializers.PrimaryKeyRelatedField(
        many=True,
        queryset=Permission.objects.all()
    )

    class Meta:
        model = Group
        fields = ['url', 'id', 'name', 'permissions']


class UserSerializer(serializers.ModelSerializer):
    groups = serializers.PrimaryKeyRelatedField(many=True, queryset=Group.objects.all())
    user_permissions = serializers.PrimaryKeyRelatedField(many=True, queryset=Permission.objects.all())

    class Meta:
        model = User
        fields = [
            'url',
            'id',
            'groups',
            'user_permissions',
            'email',
            'username',
            'password',
            'is_active',
            'is_staff',
            'is_superuser',
            'date_joined',
            'last_login',
        ]
        extra_kwargs = {
            "password": {"write_only": True},
            "date_joined": {"read_only": True},
            "last_login": {"read_only": True},
            "is_superuser": {"read_only": True}
        }

    def create(self, validated_data):
        user = User.objects.create(**validated_data)
        user.set_password(validated_data['password'])
        user.is_staff = True
        user.save()



from django.core.management.base import BaseCommand
from django.db import connection
from django.db.utils import OperationalError
from django.conf import settings


class Command(BaseCommand):
    help = "Check the existing database connection"

    def print_notice(self, message):
        return self.stdout.write(self.style.NOTICE(message))

    def handle(self, *args, **options):
        default_db = settings.DATABASES['default']
        self.print_notice('Connecting database...')
        self.print_notice("DSN for database 'default' with engine '{}'".format(default_db['ENGINE']))
        self.print_notice("host='{}' port='{}' dbname='{}' username='{}' password='{}' ".format(
            default_db['HOST'],
            default_db['PORT'],
            default_db['NAME'],
            default_db['USER'],
            default_db['PASSWORD']
        ))
        try:
            connection.ensure_connection()
            self.stdout.write(self.style.SUCCESS('Database connection available!!!'))
        except OperationalError as error:
            self.stdout.write(self.style.ERROR('Database configuration failed!!!, {}'.format(error)))
